#include "playback.h"

GltPlayback::GltPlayback(GltNoteTerminal &terminal)
: _terminal(terminal), 
  _record(false), 
  _play(false), 
  _pos(0)
{
}

GltPlayback::GltPlayback(GltNoteTerminal &terminal,unsigned char *buffer,int size)
: _terminal(terminal), 
  _record(false), 
  _play(true), 
  _pos(0)
{
	int i=0;
	while (i<size)
	{
		if (buffer[i]==0)
			break;
		else
			i++;

		if (buffer[i]==0)
			i += 5;
		else
			i++;
	}

	_buffer.insert(_buffer.begin(),buffer,buffer+i);
}

GltPlayback::~GltPlayback()
{
}

void 
GltPlayback::record()  
{ 
	flush();
	_record = true;  
	_play   = false; 
	_timer.start();
}

void 
GltPlayback::play()
{ 
	_record = false; 
	_play   = true;  
	_pos    = 0; 
	_terminal.clear(); 
}

void 
GltPlayback::clear()
{
	_record = false;  
	_play   = false; 
	_pos    = 0; 
	_terminal.clear(); 
	_buffer.clear(); 
}

void 
GltPlayback::flush()
{
	_record = false; 
	_play = true;  
	while (tick());
}

void 
GltPlayback::OnKeyboard(unsigned char key)
{
	if (_record && key!=0)
	{
		// Time is stored as tenths of a second
		unsigned int time = (unsigned int) (_timer.elapsed() * 10);

		// Ignore time for the first keypress
		if (_buffer.size()==0)
			time = 1;

		// Clamp the time to the range of an unsigned char
		time = CLAMP(time,1u,255u);

		// Store the time interval and keypress
		_buffer.push_back(time);
		_buffer.push_back(key);

		// Restart the timer
		_timer.start();
	}

	if (!_play)
		_terminal.OnKeyboard(key);
}

void 
GltPlayback::OnSpecial(int key)
{
	if (_record)
	{
		// Time is stored as tenths of a second
		unsigned int time = (unsigned int) (_timer.elapsed() * 10);

		// Ignore time for the first keypress
		if (_buffer.size()==0)
			time = 1;

		// Clamp the time to the range of an unsigned char
		time = CLAMP(time,1u,255u);

		// Store the time and full integer code
		_buffer.push_back(time);
		_buffer.push_back(0);
		_buffer.push_back((key>>24)&0xff);
		_buffer.push_back((key>>16)&0xff);
		_buffer.push_back((key>>8)&0xff);
		_buffer.push_back(key&0xff);

		// Restart the timer
		_timer.start();
	}

	if (!_play)
		_terminal.OnSpecial(key);
}

//

unsigned int 
GltPlayback::tick()
{
	unsigned int time = 0;

	if (!_record && _pos<_buffer.size() && _buffer[_pos]!=0)
	{
		time = _buffer[_pos++]*100;
		const unsigned char key = _buffer[_pos++];
		if (key==0)
		{
			const int a = _buffer[_pos++];
			const int b = _buffer[_pos++];
			const int c = _buffer[_pos++];
			const int d = _buffer[_pos++];
			_terminal.OnSpecial(a<<24 | b<<16 | c<<8 | d);
		}
		else
			_terminal.OnKeyboard(key);
	}

	return time;
}

void 
GltPlayback::copy(void *buffer,int size)
{
	memset(buffer,0,size);
	memcpy(buffer,&_buffer[0],MIN(_buffer.size(),size-6));
}


